@@ -19,7 +19,8 @@ typedef enum {
   CLUTTER_DEBUG_BACKEND         = 1 << 9,
   CLUTTER_DEBUG_SCHEDULER       = 1 << 10,
   CLUTTER_DEBUG_SCRIPT          = 1 << 11,
-  CLUTTER_DEBUG_SHADER          = 1 << 12
+  CLUTTER_DEBUG_SHADER          = 1 << 12,
+  CLUTTER_DEBUG_MULTISTAGE      = 1 << 13
 } ClutterDebugFlag;
 
 #ifdef CLUTTER_ENABLE_DEBUG
