@@ -287,14 +287,14 @@ _clutter_id_to_color (guint id, ClutterColor *col)
   ctx = clutter_context_get_default ();
 
   /* compute the numbers we'll store in the components */
-  red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used)) 
+  red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
                 & (0xff >> (8-ctx->fb_r_mask_used));
   green = (id >> ctx->fb_b_mask_used) & (0xff >> (8-ctx->fb_g_mask_used));
   blue  = (id)  & (0xff >> (8-ctx->fb_b_mask_used));
 
   /* shift left bits a bit and add one, this circumvents
    * at least some potential rounding errors in GL/GLES
-   * driver / hw implementation. 
+   * driver / hw implementation.
    */
   if (ctx->fb_r_mask_used != ctx->fb_r_mask)
     red = red * 2 + 1;
@@ -303,10 +303,10 @@ _clutter_id_to_color (guint id, ClutterColor *col)
   if (ctx->fb_b_mask_used != ctx->fb_b_mask)
     blue  = blue  * 2 + 1;
 
-  /* shift up to be full 8bit values */ 
-  red   = red   << (8 - ctx->fb_r_mask);
-  green = green << (8 - ctx->fb_g_mask);
-  blue  = blue  << (8 - ctx->fb_b_mask);
+  /* shift up to be full 8bit values */
+  red   = (red   << (8 - ctx->fb_r_mask)) | (0xff >> (ctx->fb_r_mask + 1));
+  green = (green << (8 - ctx->fb_g_mask)) | (0xff >> (ctx->fb_g_mask + 1));
+  blue  = (blue  << (8 - ctx->fb_b_mask)) | (0xff >> (ctx->fb_b_mask + 1));
 
   col->red   = red;
   col->green = green;
@@ -314,8 +314,8 @@ _clutter_id_to_color (guint id, ClutterColor *col)
   col->alpha = 0xff;
 }
 
-guint 
-_clutter_pixel_to_id (guchar pixel[4])                 
+guint
+_clutter_pixel_to_id (guchar pixel[4])
 {
   ClutterMainContext *ctx;
   gint  red, green, blue;
@@ -333,12 +333,12 @@ _clutter_pixel_to_id (guchar pixel[4])
   /* divide potentially by two if 'fuzzy' */
   red   = red   >> (ctx->fb_r_mask - ctx->fb_r_mask_used);
   green = green >> (ctx->fb_g_mask - ctx->fb_g_mask_used);
-  blue  = blue  >> (ctx->fb_b_mask - ctx->fb_b_mask_used);  
+  blue  = blue  >> (ctx->fb_b_mask - ctx->fb_b_mask_used);
 
   /* combine the correct per component values into the final id */
-  id =  blue + (green <<  ctx->fb_b_mask_used) 
+  id =  blue + (green <<  ctx->fb_b_mask_used)
           + (red << (ctx->fb_b_mask_used + ctx->fb_g_mask_used));
-  
+
   return id;
 }
 
@@ -353,7 +353,7 @@ _clutter_do_pick (ClutterStage   *stage,
   GLint               viewport[4];
   ClutterColor        white = { 0xff, 0xff, 0xff, 0xff };
   guint32             id;
-  
+
   context = clutter_context_get_default ();
 
   _clutter_backend_ensure_context (context->backend, stage);
@@ -395,25 +395,6 @@ _clutter_do_pick (ClutterStage   *stage,
   return clutter_get_actor_by_gid (id);
 }
 
-static void
-clutter_context_free (ClutterMainContext *context)
-{
-  /* this will take care of destroying the stage */
-  g_object_unref (context->backend);
-  context->backend = NULL;
-
-  clutter_id_pool_free (context->id_pool);
-  context->id_pool = NULL;
-
-  g_object_unref (context->font_map);
-  context->font_map = NULL;
-
-  /* XXX: The cleaning up of the event queue should be moved here from
-          the backend base class. */
-
-  g_free (context);
-}
-
 PangoContext *
 _clutter_context_create_pango_context (ClutterMainContext *self)
 {
@@ -469,9 +450,11 @@ clutter_main_level (void)
 void
 clutter_main (void)
 {
-  ClutterMainContext *context = CLUTTER_CONTEXT ();
   GMainLoop *loop;
 
+  /* Make sure there is a context */
+  CLUTTER_CONTEXT ();
+
   if (!clutter_is_initialized)
     {
       g_warning ("Called clutter_main() but Clutter wasn't initialised.  "
@@ -506,11 +489,6 @@ clutter_main (void)
 
   clutter_main_loop_level--;
 
-  if (clutter_main_loop_level == 0)
-    {
-      clutter_context_free (context);
-    }
-
   CLUTTER_MARK ();
 }
 
@@ -645,45 +623,85 @@ clutter_threads_dispatch_free (gpointer data)
  * @notify: functio to call when the idle source is removed
  *
  * Adds a function to be called whenever there are no higher priority
- * events pending.  If the function returns %FALSE it is automatically
+ * events pending. If the function returns %FALSE it is automatically
  * removed from the list of event sources and will not be called again.
  *
- * This variant of g_idle_add_full() calls @function with the Clutter lock
- * held. It can be thought of a MT-safe version for Clutter actors for the
- * following use case, where you have to worry about idle_callback()
- * running in thread A and accessing @self after it has been finalized
- * in thread B:
+ * This function can be considered a thread-safe variant of g_idle_add_full():
+ * it will call @function while holding the Clutter lock. It is logically
+ * equivalent to the following implementation:
  *
- * <informalexample><programlisting>
+ * |[
  * static gboolean
- * idle_callback (gpointer data)
+ * idle_safe_callback (gpointer data)
  * {
- *    // clutter_threads_enter(); would be needed for g_idle_add()
+ *    SafeClosure *closure = data;
+ *    gboolean res = FALSE;
  *
- *    SomeActor *self = data;
- *    /<!-- -->* do stuff with self *<!-- -->/
+ *    /&ast; mark the critical section &ast;/
  *
- *    self->idle_id = 0;
+ *    clutter_threads_enter();
  *
- *    // clutter_threads_leave(); would be needed for g_idle_add()
- *    return FALSE;
+ *    /&ast; the callback does not need to acquire the Clutter
+ *     &ast; lock itself, as it is held by the this proxy handler
+ *     &ast;/
+ *    res = closure->callback (closure->data);
+ *
+ *    clutter_threads_leave();
+ *
+ *    return res;
  * }
- * static void
- * some_actor_do_stuff_later (SomeActor *self)
+ * static gulong
+ * add_safe_idle (GSourceFunc callback,
+ *                gpointer    data)
  * {
- *    self->idle_id = clutter_threads_add_idle (idle_callback, self)
- *    // using g_idle_add() here would require thread protection in the callback
+ *   SafeClosure *closure = g_new0 (SafeClosure, 1);
+ *
+ *   closure-&gt;callback = callback;
+ *   closure-&gt;data = data;
+ *
+ *   return g_add_idle_full (G_PRIORITY_DEFAULT_IDLE,
+ *                           idle_safe_callback,
+ *                           closure,
+ *                           g_free)
+ * }
+ *]|
+ *
+ * This function should be used by threaded applications to make sure
+ * that @func is emitted under the Clutter threads lock and invoked
+ * from the same thread that started the Clutter main loop. For instance,
+ * it can be used to update the UI using the results from a worker
+ * thread:
+ *
+ * |[
+ * static gboolean
+ * update_ui (gpointer data)
+ * {
+ *   SomeClosure *closure = data;
+ *
+ *   /&ast; it is safe to call Clutter API from this function because
+ *    &ast; it is invoked from the same thread that started the main
+ *    &ast; loop and under the Clutter thread lock
+ *    &ast;/
+ *   clutter_label_set_text (CLUTTER_LABEL (closure-&gt;label),
+ *                           closure-&gt;text);
+ *
+ *   g_object_unref (closure-&gt;label);
+ *   g_free (closure);
+ *
+ *   return FALSE;
  * }
  *
- * static void
- * some_actor_finalize (GObject *object)
- * {
- *    SomeActor *self = SOME_ACTOR (object);
- *    if (self->idle_id)
- *      g_source_remove (self->idle_id);
- *    G_OBJECT_CLASS (parent_class)->finalize (object);
- * }
- * </programlisting></informalexample>
+ *   /&ast; within another thread &ast;/
+ *   closure = g_new0 (SomeClosure, 1);
+ *   /&ast; always take a reference on GObject instances &ast;/
+ *   closure-&gt;label = g_object_ref (my_application-&gt;label);
+ *   closure-&gt;text = g_strdup (processed_text_to_update_the_label);
+ *
+ *   clutter_threads_add_idle_full (G_PRIORITY_HIGH_IDLE,
+ *                                  update_ui,
+ *                                  closure,
+ *                                  NULL);
+ * ]|
  *
  * Return value: the ID (greater than 0) of the event source.
  *
@@ -714,7 +732,8 @@ clutter_threads_add_idle_full (gint           priority,
  * @func: function to call
  * @data: data to pass to the function
  *
- * Simple wrapper around clutter_threads_add_idle_full()
+ * Simple wrapper around clutter_threads_add_idle_full() using the
+ * default priority.
  *
  * Return value: the ID (greater than 0) of the event source.
  *
@@ -740,21 +759,21 @@ clutter_threads_add_idle (GSourceFunc func,
  * @data: data to pass to the function
  * @notify: function to call when the timeout source is removed
  *
- * Sets a function to be called at regular intervals holding the Clutter lock,
- * with the given priority.  The function is called repeatedly until it
- * returns %FALSE, at which point the timeout is automatically destroyed
- * and the function will not be called again.  The @notify function is
- * called when the timeout is destroyed.  The first call to the
- * function will be at the end of the first @interval.
+ * Sets a function to be called at regular intervals holding the Clutter
+ * threads lock, with the given priority. The function is called repeatedly
+ * until it returns %FALSE, at which point the timeout is automatically
+ * removed and the function will not be called again. The @notify function
+ * is called when the timeout is removed.
  *
- * Note that timeout functions may be delayed, due to the processing of other
- * event sources. Thus they should not be relied on for precise timing.
- * After each call to the timeout function, the time of the next
- * timeout is recalculated based on the current time and the given interval
- * (it does not try to 'catch up' time lost in delays).
+ * The first call to the function will be at the end of the first @interval.
  *
- * This variant of g_timeout_add_full() can be thought of a MT-safe version
- * for Clutter actors. See also clutter_threads_add_idle_full().
+ * It is important to note that, due to how the Clutter main loop is
+ * implemented, the timing will not be accurate and it will not try to
+ * "keep up" with the interval. A more reliable source is available
+ * using clutter_threads_add_frame_source_full(), which is also internally
+ * used by #ClutterTimeline.
+ *
+ * See also clutter_threads_add_idle_full().
  *
  * Return value: the ID (greater than 0) of the event source.
  *
@@ -816,12 +835,11 @@ clutter_threads_add_timeout (guint       interval,
  * @data: data to pass to the function
  * @notify: function to call when the timeout source is removed
  *
- * Sets a function to be called at regular intervals holding the Clutter lock,
- * with the given priority.  The function is called repeatedly until it
- * returns %FALSE, at which point the timeout is automatically destroyed
- * and the function will not be called again.  The @notify function is
- * called when the timeout is destroyed.  The first call to the
- * function will be at the end of the first @interval.
+ * Sets a function to be called at regular intervals holding the Clutter
+ * threads lock, with the given priority. The function is called repeatedly
+ * until it returns %FALSE, at which point the timeout is automatically
+ * removed and the function will not be called again. The @notify function
+ * is called when the timeout is removed.
  *
  * This function is similar to clutter_threads_add_timeout_full()
  * except that it will try to compensate for delays. For example, if
@@ -833,9 +851,8 @@ clutter_threads_add_timeout (guint       interval,
  * to invoke the function multiple times to catch up missing frames if
  * @func takes more than @interval ms to execute.
  *
- * This variant of clutter_frame_source_add_full() can be thought of a
- * MT-safe version for Clutter actors.
-
+ * See also clutter_threads_add_idle_full().
+ *
  * Return value: the ID (greater than 0) of the event source.
  *
  * Since: 0.8
@@ -939,7 +956,6 @@ clutter_context_get_default (void)
   if (G_UNLIKELY(!ClutterCntx))
     {
       ClutterMainContext *ctx;
-      gdouble resolution;
 
       ClutterCntx = ctx = g_new0 (ClutterMainContext, 1);
       ctx->backend = g_object_new (_clutter_backend_impl_get_type (), NULL);
@@ -951,13 +967,6 @@ clutter_context_get_default (void)
       ctx->timer          =  g_timer_new ();
       g_timer_start (ctx->timer);
 #endif
-
-      ctx->font_map = PANGO_CLUTTER_FONT_MAP (pango_clutter_font_map_new ());
-
-      resolution = clutter_backend_get_resolution (ctx->backend);
-      pango_clutter_font_map_set_resolution (ctx->font_map, resolution);
-
-      pango_clutter_font_map_set_use_mipmapping (ctx->font_map, TRUE);
     }
 
   return ClutterCntx;
@@ -1016,6 +1025,117 @@ clutter_arg_no_debug_cb (const char *key,
 }
 #endif /* CLUTTER_ENABLE_DEBUG */
 
+GQuark
+clutter_init_error_quark (void)
+{
+  return g_quark_from_static_string ("clutter-init-error-quark");
+}
+
+static ClutterInitError
+clutter_init_real (GError **error)
+{
+  ClutterMainContext *ctx;
+  ClutterActor *stage;
+  gdouble resolution;
+  ClutterBackend *backend;
+
+  /* Note, creates backend if not already existing, though parse args will
+   * have likely created it
+   */
+  ctx = clutter_context_get_default ();
+  backend = ctx->backend;
+
+  if (!ctx->options_parsed)
+    {
+      g_set_error (error, CLUTTER_INIT_ERROR,
+		   CLUTTER_INIT_ERROR_INTERNAL,
+		   "When using clutter_get_option_group_without_init() "
+		   "you must parse options before calling clutter_init()");
+
+      return CLUTTER_INIT_ERROR_INTERNAL;
+    }
+
+  /*
+   * Call backend post parse hooks.
+   */
+  if (CLUTTER_BACKEND_GET_CLASS (backend)->post_parse)
+    if (!CLUTTER_BACKEND_GET_CLASS (backend)->post_parse (backend, error))
+      return CLUTTER_INIT_ERROR_BACKEND;
+
+  /*
+   * Resolution requires display to be open, so can only be queried after
+   * the post_parse hooks run.
+   */
+  ctx->font_map = PANGO_CLUTTER_FONT_MAP (pango_clutter_font_map_new ());
+
+  resolution = clutter_backend_get_resolution (ctx->backend);
+  pango_clutter_font_map_set_resolution (ctx->font_map, resolution);
+
+  pango_clutter_font_map_set_use_mipmapping (ctx->font_map, TRUE);
+
+  /* Stage will give us a GL Context etc */
+  stage = clutter_stage_get_default ();
+  if (!stage)
+    {
+      if (error)
+        g_set_error (error, CLUTTER_INIT_ERROR,
+                     CLUTTER_INIT_ERROR_INTERNAL,
+                     "Unable to create the default stage");
+      else
+        g_critical ("Unable to create the default stage");
+
+      return CLUTTER_INIT_ERROR_INTERNAL;
+    }
+
+  clutter_actor_realize (stage);
+
+  if (!CLUTTER_ACTOR_IS_REALIZED (stage))
+    {
+      if (error)
+        g_set_error (error, CLUTTER_INIT_ERROR,
+                     CLUTTER_INIT_ERROR_INTERNAL,
+                     "Unable to realize the default stage");
+      else
+        g_critical ("Unable to realize the default stage");
+
+      return CLUTTER_INIT_ERROR_INTERNAL;
+    }
+
+  /* Now we can safely assume we have a valid GL context and can
+   * start issueing cogl commands
+  */
+
+  /* Figure out framebuffer masks used for pick */
+  cogl_get_bitmasks (&ctx->fb_r_mask, &ctx->fb_g_mask, &ctx->fb_b_mask, NULL);
+
+  ctx->fb_r_mask_used = ctx->fb_r_mask;
+  ctx->fb_g_mask_used = ctx->fb_g_mask;
+  ctx->fb_b_mask_used = ctx->fb_b_mask;
+
+#ifndef HAVE_CLUTTER_FRUITY
+  /* We always do fuzzy picking for the fruity backend */
+  if (g_getenv ("CLUTTER_FUZZY_PICK") != NULL)
+#endif
+    {
+      ctx->fb_r_mask_used--;
+      ctx->fb_g_mask_used--;
+      ctx->fb_b_mask_used--;
+    }
+
+  /* Initiate event collection */
+  _clutter_backend_init_events (ctx->backend);
+
+  /* finally features - will call to backend and cogl */
+  _clutter_feature_init ();
+
+  clutter_stage_set_title (CLUTTER_STAGE (stage), g_get_prgname ());
+
+  clutter_is_initialized = TRUE;
+  ctx->is_initialized = TRUE;
+
+  return CLUTTER_INIT_SUCCESS;
+}
+
 static GOptionEntry clutter_args[] = {
   { "clutter-show-fps", 0, 0, G_OPTION_ARG_NONE, &clutter_show_fps,
     "Show frames per second", NULL },
@@ -1048,7 +1168,7 @@ pre_parse_hook (GOptionContext  *context,
 
   if (clutter_is_initialized)
     return TRUE;
-  
+
   if (setlocale (LC_ALL, "") == NULL)
     g_warning ("Locale not supported by C library.\n"
                "Using the fallback 'C' locale.");
@@ -1101,7 +1221,6 @@ post_parse_hook (GOptionContext  *context,
 {
   ClutterMainContext *clutter_context;
   ClutterBackend *backend;
-  gboolean retval = FALSE;
 
   if (clutter_is_initialized)
     return TRUE;
@@ -1120,13 +1239,16 @@ post_parse_hook (GOptionContext  *context,
     }
 
   clutter_context->frame_rate = clutter_default_fps;
+  clutter_context->options_parsed = TRUE;
 
-  if (CLUTTER_BACKEND_GET_CLASS (backend)->post_parse)
-    retval = CLUTTER_BACKEND_GET_CLASS (backend)->post_parse (backend, error);
-  else
-    retval = TRUE;
+  /*
+   * If not asked to defer display setup, call clutter_init_real(),
+   * which in turn calls the backend post parse hooks.
+   */
+  if (!clutter_context->defer_display_setup)
+    return clutter_init_real (error);
 
-  return retval;
+  return TRUE;
 }
 
 /**
@@ -1137,7 +1259,25 @@ post_parse_hook (GOptionContext  *context,
  * g_option_context_add_group(), if you are using g_option_context_parse()
  * to parse your commandline arguments.
  *
- * Return value: a GOptionGroup for the commandline arguments
+ * Calling g_option_context_parse() with Clutter's #GOptionGroup will result
+ * in Clutter's initialization. That is, the following code:
+ *
+ * |[
+ *   g_option_context_set_main_group (context, clutter_get_option_group ());
+ *   res = g_option_context_parse (context, &amp;argc, &amp;argc, NULL);
+ * ]|
+ *
+ * is functionally equivalent to:
+ *
+ * |[
+ *   clutter_init (&amp;argc, &amp;argv);
+ * ]|
+ *
+ * After g_option_context_parse() on a #GOptionContext containing the
+ * Clutter #GOptionGroup has returned %TRUE, Clutter is guaranteed to be
+ * initialized.
+ *
+ * Return value: a #GOptionGroup for the commandline arguments
  *   recognized by Clutter
  *
  * Since: 0.2
@@ -1148,6 +1288,8 @@ clutter_get_option_group (void)
   ClutterMainContext *context;
   GOptionGroup *group;
 
+  clutter_base_init ();
+
   context = clutter_context_get_default ();
 
   group = g_option_group_new ("clutter",
@@ -1165,88 +1307,43 @@ clutter_get_option_group (void)
   return group;
 }
 
-GQuark
-clutter_init_error_quark (void)
+/**
+ * clutter_get_option_group_without_init:
+ *
+ * Returns a #GOptionGroup for the command line arguments recognized
+ * by Clutter. You should add this group to your #GOptionContext with
+ * g_option_context_add_group(), if you are using g_option_context_parse()
+ * to parse your commandline arguments. Unlike clutter_get_option_group(),
+ * calling g_option_context_parse() with the #GOptionGroup returned by this
+ * function requires a subsequent explicit call to clutter_init(); use this
+ * function when needing to set foreign display connection with
+ * clutter_x11_set_display(), or with gtk_clutter_init().
+ *
+ * Return value: a #GOptionGroup for the commandline arguments
+ *   recognized by Clutter
+ *
+ * Since: 0.8.2
+ */
+GOptionGroup *
+clutter_get_option_group_without_init (void)
 {
-  return g_quark_from_static_string ("clutter-init-error-quark");
-}
+  ClutterMainContext *context;
+  GOptionGroup *group;
 
-static ClutterInitError
-clutter_init_real (GError **error)
-{
-  ClutterMainContext *ctx;
-  ClutterActor *stage;
+  clutter_base_init ();
 
-  /* Note, creates backend if not already existing, though parse args will
-   * have likely created it
-   */
-  ctx = clutter_context_get_default ();
+  context = clutter_context_get_default ();
+  context->defer_display_setup = TRUE;
 
-  /* Stage will give us a GL Context etc */
-  stage = clutter_stage_get_default ();
-  if (!stage)
-    {
-      if (error)
-        g_set_error (error, CLUTTER_INIT_ERROR,
-                     CLUTTER_INIT_ERROR_INTERNAL,
-                     "Unable to create the default stage");
-      else
-        g_critical ("Unable to create the default stage");
-      return CLUTTER_INIT_ERROR_INTERNAL;
-    }
+  group = clutter_get_option_group ();
 
-  clutter_actor_realize (stage);
-
-  if (!CLUTTER_ACTOR_IS_REALIZED (stage))
-    {
-      if (error)
-        g_set_error (error, CLUTTER_INIT_ERROR,
-                     CLUTTER_INIT_ERROR_INTERNAL,
-                     "Unable to realize the default stage");
-      else
-        g_critical ("Unable to realize the default stage");
-
-      return CLUTTER_INIT_ERROR_INTERNAL;
-    }
-
-  /* Now we can safely assume we have a valid GL context and can 
-   * start issueing cogl commands
-  */
-
-  /* Figure out framebuffer masks used for pick */
-  cogl_get_bitmasks (&ctx->fb_r_mask, &ctx->fb_g_mask, &ctx->fb_b_mask, NULL);
-
-  ctx->fb_r_mask_used = ctx->fb_r_mask;
-  ctx->fb_g_mask_used = ctx->fb_g_mask;
-  ctx->fb_b_mask_used = ctx->fb_b_mask;
-
-#ifndef HAVE_CLUTTER_FRUITY
-  /* We always do fuzzy picking for the fruity backend */
-  if (g_getenv ("CLUTTER_FUZZY_PICK") != NULL)
-#endif
-    {
-      ctx->fb_r_mask_used--;
-      ctx->fb_g_mask_used--;
-      ctx->fb_b_mask_used--;
-    }
-
-  /* Initiate event collection */
-  _clutter_backend_init_events (ctx->backend);
-
-  /* finally features - will call to backend and cogl */
-  _clutter_feature_init ();
-
-  clutter_stage_set_title (CLUTTER_STAGE (stage), g_get_prgname ());
-
-  clutter_is_initialized = TRUE;
-
-  return CLUTTER_INIT_SUCCESS;
+  return group;
 }
 
 /**
  * clutter_init_with_args:
  * @argc: a pointer to the number of command line arguments
- * @argv: a pointer to the array of comman line arguments
+ * @argv: a pointer to the array of command line arguments
  * @parameter_string: a string which is displayed in the
  *   first line of <option>--help</option> output, after
  *   <literal><replaceable>programname</replaceable> [OPTION...]</literal>
@@ -1282,34 +1379,49 @@ clutter_init_with_args (int            *argc,
   GOptionContext *context;
   GOptionGroup *group;
   gboolean res;
+  ClutterMainContext *ctx;
 
   if (clutter_is_initialized)
     return CLUTTER_INIT_SUCCESS;
 
   clutter_base_init ();
 
-  if (argc && *argc > 0 && *argv)
-    g_set_prgname ((*argv)[0]);
+  ctx = clutter_context_get_default ();
 
-  group   = clutter_get_option_group ();
-  context = g_option_context_new (parameter_string);
+  if (!ctx->defer_display_setup)
+    {
+      if (argc && *argc > 0 && *argv)
+	g_set_prgname ((*argv)[0]);
 
-  g_option_context_add_group (context, group);
+      group   = clutter_get_option_group ();
+      context = g_option_context_new (parameter_string);
 
-  if (entries)
-    g_option_context_add_main_entries (context, entries, translation_domain);
+      g_option_context_add_group (context, group);
 
-  res = g_option_context_parse (context, argc, argv, error);
-  g_option_context_free (context);
+      if (entries)
+	g_option_context_add_main_entries (context, entries, translation_domain);
 
-  /* if res is FALSE, the error is filled for
-   * us by g_option_context_parse()
-   */
-  if (!res)
-    return CLUTTER_INIT_ERROR_INTERNAL;
+      res = g_option_context_parse (context, argc, argv, error);
+      g_option_context_free (context);
 
-  /* Do the real work.. */
-  return clutter_init_real (error);
+      /* if res is FALSE, the error is filled for
+       * us by g_option_context_parse()
+       */
+      if (!res)
+	{
+	  /* if there has been an error in the initialization, the
+	   * error id will be preserved inside the GError code
+	   */
+	  if (error && *error)
+	    return (*error)->code;
+	  else
+	    return CLUTTER_INIT_ERROR_INTERNAL;
+	}
+
+      return CLUTTER_INIT_SUCCESS;
+    }
+  else
+    return clutter_init_real (error);
 }
 
 static gboolean
@@ -1335,8 +1447,12 @@ clutter_parse_args (int    *argc,
 
   if (!g_option_context_parse (option_context, argc, argv, &error))
     {
-      g_warning ("%s", error->message);
-      g_error_free (error);
+      if (error)
+	{
+	  g_warning ("%s", error->message);
+	  g_error_free (error);
+	}
+
       ret = FALSE;
     }
 
@@ -1361,24 +1477,34 @@ ClutterInitError
 clutter_init (int    *argc,
               char ***argv)
 {
+  ClutterMainContext *ctx;
+  GError *error = NULL;
+
   if (clutter_is_initialized)
     return CLUTTER_INIT_SUCCESS;
 
   clutter_base_init ();
 
-  if (argc && *argc > 0 && *argv)
-    g_set_prgname ((*argv)[0]);
+  ctx = clutter_context_get_default ();
 
-  /* parse_args will trigger backend creation and things like
-   * DISPLAY connection etc.
-  */
-  if (clutter_parse_args (argc, argv) == FALSE)
+  if (!ctx->defer_display_setup)
     {
-      CLUTTER_NOTE (MISC, "failed to parse arguments.");
-      return CLUTTER_INIT_ERROR_INTERNAL;
-    }
+      if (argc && *argc > 0 && *argv)
+	g_set_prgname ((*argv)[0]);
 
-  return clutter_init_real (NULL);
+      /* parse_args will trigger backend creation and things like
+       * DISPLAY connection etc.
+       */
+      if (clutter_parse_args (argc, argv) == FALSE)
+	{
+	  CLUTTER_NOTE (MISC, "failed to parse arguments.");
+	  return CLUTTER_INIT_ERROR_INTERNAL;
+	}
+
+      return CLUTTER_INIT_SUCCESS;
+    }
+  else
+    return clutter_init_real (&error);
 }
 
 gboolean
@@ -1445,7 +1571,7 @@ event_click_count_generate (ClutterEvent *event)
           }
 
         /* store time and position for this click for comparison with
-         * next event 
+         * next event
          */
         previous_time = event->button.time;
         previous_x    = event->button.x;
@@ -1652,7 +1778,7 @@ generate_enter_leave_events (ClutterEvent *event)
 
   if (last_actor && last_actor != motion_current_actor)
     {
-      g_signal_handlers_disconnect_by_func 
+      g_signal_handlers_disconnect_by_func
                        (last_actor,
                         G_CALLBACK (unset_motion_last_actor),
                         event->motion.device);
@@ -1724,7 +1850,7 @@ clutter_do_event (ClutterEvent *event)
             else
               clutter_actor_destroy (stage);
           }
-        
+
         break;
 
       case CLUTTER_KEY_PRESS:
@@ -1777,7 +1903,16 @@ clutter_do_event (ClutterEvent *event)
                   delta,
                   event->any.time);
 
-            if (event->any.time < (local_motion_time + delta))
+            /* we need to guard against roll-overs and the
+             * case where the time is rolled backwards and
+             * the backend is not ensuring a monotonic clock
+             * for the events.
+             *
+             * see:
+             *   http://bugzilla.openedhand.com/show_bug.cgi?id=1130
+             */
+            if (event->any.time >= local_motion_time &&
+                event->any.time < (local_motion_time + delta))
               break;
             else
               local_motion_time = event->any.time;
@@ -1800,13 +1935,13 @@ clutter_do_event (ClutterEvent *event)
             /* global grabs */
             if (context->pointer_grab_actor != NULL)
               {
-                clutter_actor_event (context->pointer_grab_actor, 
+                clutter_actor_event (context->pointer_grab_actor,
                                      event, FALSE);
                 break;
               }
             else if (device != NULL && device->pointer_grab_actor != NULL)
               {
-                clutter_actor_event (device->pointer_grab_actor, 
+                clutter_actor_event (device->pointer_grab_actor,
                                      event, FALSE);
                 break;
               }
@@ -2013,7 +2148,7 @@ on_pointer_grab_weak_notify (gpointer data,
 {
   ClutterInputDevice *dev = (ClutterInputDevice *)data;
   ClutterMainContext *context;
-  
+
   context = clutter_context_get_default ();
 
   if (dev)
@@ -2318,7 +2453,7 @@ clutter_set_motion_events_frequency (guint frequency)
  * renderer. This will free up some memory and GL texture
  * resources. The cache will be automatically refilled as more text is
  * drawn.
- * 
+ *
  * Since: 0.8
  */
 void
@@ -2367,7 +2502,7 @@ clutter_get_use_mipmapped_text (void)
   if (font_map)
     return pango_clutter_font_map_get_use_mipmapping (font_map);
 
-  return FALSE;    
+  return FALSE;
 }
 
 /**
@@ -2389,8 +2524,8 @@ clutter_get_input_device_for_id (gint id)
 
   context = clutter_context_get_default ();
 
-  for (item = context->input_devices; 
-       item != NULL; 
+  for (item = context->input_devices;
+       item != NULL;
        item = item->next)
   {
     device = item->data;
