@@ -91,14 +91,16 @@ struct _ClutterMainContext
                                           system backend */
   ClutterStageManager *stage_manager;  /* stages */
   GQueue          *events_queue;       /* the main event queue */
-  
+
   guint            is_initialized : 1;  
+  guint            motion_events_per_actor : 1;/* set for enter/leave events */
+  guint            defer_display_setup : 1;
+  guint            options_parsed : 1;
+
   GTimer          *timer;	       /* Used for debugging scheduler */
 
   ClutterPickMode  pick_mode;          /* Indicates pick render mode   */
 
-  guint            motion_events_per_actor : 1;/* set for enter/leave events */
-
   guint            motion_frequency;   /* Motion events per second */
   gint             num_reactives;      /* Num of reactive actors */
 
@@ -210,6 +212,9 @@ gboolean _clutter_boolean_handled_accumulator (GSignalInvocationHint *ihint,
                                                const GValue          *handler_return,
                                                gpointer               dummy);
 
+void _clutter_actor_apply_modelview_transform_recursive (ClutterActor *self,
+						       ClutterActor *ancestor);
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_PRIVATE_H */
