@@ -357,7 +357,7 @@ clutter_stage_set_property (GObject      *object,
       if (CLUTTER_ACTOR_IS_REALIZED (actor))
         {
           /* Backend needs to check this prop and handle accordingly
-           * in realise. 
+           * in realise.
            * FIXME: More 'obvious' implementation needed?
           */
           clutter_actor_unrealize (actor);
@@ -455,14 +455,14 @@ clutter_stage_dispose (GObject *object)
   ClutterStagePrivate *priv = stage->priv;
   ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
 
+  clutter_actor_unrealize (CLUTTER_ACTOR (object));
+
   if (priv->update_idle)
     {
       g_source_remove (priv->update_idle);
       priv->update_idle = 0;
     }
 
-  clutter_actor_unrealize (CLUTTER_ACTOR (object));
-  
   _clutter_stage_manager_remove_stage (stage_manager, stage);
 
   if (priv->impl)
@@ -478,6 +478,10 @@ clutter_stage_dispose (GObject *object)
 static void
 clutter_stage_finalize (GObject *object)
 {
+  ClutterStage *stage = CLUTTER_STAGE (object);
+
+  g_free (stage->priv->title);
+  
   G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
 }
 
@@ -566,6 +570,23 @@ clutter_stage_class_init (ClutterStageClass *klass)
 			 "The color of the main stage",
 			 CLUTTER_TYPE_COLOR,
 			 CLUTTER_PARAM_READWRITE));
+
+  /**
+   * ClutterStage:perspective:
+   *
+   * The parameters used for the perspective projection from 3D
+   * coordinates to 2D
+   *
+   * Since: 0.8.2
+   */
+  g_object_class_install_property
+    (gobject_class, PROP_PERSPECTIVE,
+     g_param_spec_boxed ("perspective",
+			 "Perspective",
+			 "Perspective projection parameters",
+			 CLUTTER_TYPE_PERSPECTIVE,
+			 CLUTTER_PARAM_READWRITE));
+
   /**
    * ClutterStage:title:
    *
@@ -803,7 +824,7 @@ clutter_stage_get_color (ClutterStage *stage,
  * @stage: A #ClutterStage
  * @perspective: A #ClutterPerspective
  *
- * Set the stage perspective. This is the fixed point version of 
+ * Set the stage perspective. This is the fixed point version of
  * clutter_stage_set_perspective().
  **/
 void
@@ -830,7 +851,7 @@ clutter_stage_set_perspectivex (ClutterStage       *stage,
  * @stage: A #ClutterStage
  * @perspective: return location for a #ClutterPerspective
  *
- * Retrieves the stage perspective. This is the fixed point version of 
+ * Retrieves the stage perspective. This is the fixed point version of
  * clutter_stage_get_perspective().
  */
 void
@@ -1100,7 +1121,7 @@ clutter_stage_hide_cursor (ClutterStage *stage)
       iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
       if (iface->set_cursor_visible)
         {
-          priv->is_cursor_visible = TRUE;
+          priv->is_cursor_visible = FALSE;
 
           iface->set_cursor_visible (impl, FALSE);
 
@@ -1122,7 +1143,7 @@ clutter_stage_hide_cursor (ClutterStage *stage)
  * Makes a screenshot of the stage in RGBA 8bit data, returns a
  * linear buffer with @width * 4 as rowstride.
  *
- * The alpha data contained in the returned buffer is driver-dependent, 
+ * The alpha data contained in the returned buffer is driver-dependent,
  * and not guaranteed to hold any sensible value.
  *
  * Return value: a pointer to newly allocated memory with the buffer
@@ -1150,7 +1171,7 @@ clutter_stage_read_pixels (ClutterStage *stage,
   g_return_val_if_fail (x >= 0 && y >= 0, NULL);
 
   /* Force a redraw of the stage before reading back pixels */
-  clutter_stage_paint (CLUTTER_ACTOR (stage));
+  clutter_redraw (stage);
   clutter_stage_ensure_current (stage);
 
   glGetIntegerv (GL_VIEWPORT, viewport);
@@ -1170,6 +1191,21 @@ clutter_stage_read_pixels (ClutterStage *stage,
   pixels  = g_malloc (height * rowstride);
   temprow = g_malloc (rowstride);
 
+  /* Setup the pixel store parameters that may have been changed by
+     Cogl */
+  glPixelStorei (GL_PACK_ALIGNMENT, 4);
+#ifdef HAVE_COGL_GL
+  glPixelStorei (GL_PACK_ROW_LENGTH, 0);
+  glPixelStorei (GL_PACK_SKIP_PIXELS, 0);
+  glPixelStorei (GL_PACK_SKIP_ROWS, 0);
+#endif /* HAVE_COGL_GL */
+
+  /* The y co-ordinate should be given in OpenGL's coordinate system
+     so 0 is the bottom row */
+  y = stage_height - 1 - y - height;
+
+  glFinish ();
+
   /* check whether we need to read into a smaller temporary buffer */
   glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
 
@@ -1180,7 +1216,7 @@ clutter_stage_read_pixels (ClutterStage *stage,
         {
           memcpy (temprow,
                   pixels + y * rowstride, rowstride);
-          memcpy (pixels + y * rowstride, 
+          memcpy (pixels + y * rowstride,
                   pixels + (height - y - 1) * rowstride, rowstride);
           memcpy (pixels + (height - y - 1) * rowstride,
                   temprow,
